<?php

namespace Bydedi\Security\Driver;

use Bydedi\Database\ConnectionInterface;
use Bydedi\Database\Query;
use Bydedi\Security\Role;
use Bydedi\Security\Permission;

/**
 * Class DediDriver
 * @package Dedi\Security\Driver
 *
 * Permet de sauvegarder les roles / permissions en bdd et de les retrouver
 *
 */
class BydediDriver implements DriverInterface
{

    protected $connection;

    public function __construct(ConnectionInterface $connection)
    {
        $this->connection = $connection;
    }

    /**
     * @return Role[]
     */
    public function getRoles()
    {

        $query = new Query();
        $query->select()->from('role');
        $results = $this->connection->getAll($query->build());

        $roles = [];

        foreach ($results as $result) {

            $role = new Role();
            $role->id = $result['id'];
            $role->name = $result['name'];
            $roles[] = $role;
        }

        return $roles;
    }

    /**
     * @param Role $role
     * @return Permission[]
     */
    public function getPermissions(Role $role = null)
    {

        $query = new Query();
        $query->select('p.*')->from('permission');
        if (is_null($role) == false) {
            $query->leftJoin('role_permission', 'p.id', 'rp.permission_id')
                ->where('role_id', $role->id);
        }
        $results = $this->connection->getAll($query->build());

        $permissions = [];

        foreach ($results as $result) {

            $permission = new Permission();
            $permission->id = $result['id'];
            $permission->code = $result['code'];
            $permission->name = $result['name'];
            $permissions[] = $permission;
        }

        return $permissions;
    }

    public function addPermission(Role $role, Permission $permission)
    {
        // TODO: Implement addPermission() method.
    }

    public function deletePermission(Role $role, Permission $permission)
    {
        // TODO: Implement deletePermission() method.
    }

    public function findPermissionByCode($permissionCode)
    {
        $query = new Query();
        $query->select()->from('permission')
            ->where('code', $permissionCode);

        $result = $this->connection->getOne($query->build());

        $permission = null;
        if (!empty($result)) {
            $permission = new Permission();
            $permission->id = $result['id'];
            $permission->code = $result['code'];
            $permission->name = $result['name'];
        }

        return $permission;
    }

    public function findRoleByName($roleName)
    {
        $query = new Query();
        $query->select()->from('role')
            ->where('name', $roleName);

        $result = $this->connection->getOne($query->build());

        $role = null;
        if (!empty($result)) {
            $role = new Role();
            $role->id = $result['id'];
            $role->name = $result['name'];
            $role->description = $result['description'];
        }

        return $role;
    }
}
