<?php

namespace Bydedi\Security\Model;

use Bydedi\Database\ActiveRecord\Exception\ModelNotLoadedException;
use Bydedi\Database\ActiveRecord\Model;
use Bydedi\Database\Query;
use Bydedi\Security\UserInterface;

class User extends Model implements UserInterface
{
    protected $table = "user";

    public function findByLogin($login)
    {
        $result = $this->getOne([
            'login' => $this->connection->escape($login),
        ]);

        if (!is_null($result)) {
            $user = new static();
            $user->load($result['id']);
            return $user;
        }

        return $result;
    }

    public function findById($id)
    {
        return new static($id);
    }

    /**
     * @return array
     *
     * @throws ModelNotLoadedException
     * @throws \Exception
     */
    public function getRoles()
    {
        if ($this->isLoaded() == false) {
            throw new ModelNotLoadedException(
                sprintf('the model %s must be loaded to use "%s"', get_class($this), __FUNCTION__)
            );
        }

        $query = new Query();
        $query->select()->from('user_role')
            ->leftJoin('role', 'ur.role_id', 'r.id')
            ->where('user_id', $this->id);

        return $this->connection->getAll($query->build());
    }

    public function getRolesNames()
    {
        $roles = $this->getRoles();

        $rolesNames = [];
        foreach ($roles as $role) {
            $rolesNames[] = $role['name'];
        }

        return $rolesNames;
    }

    public function encryptPassword($plain)
    {
        $password = '';

        for ($i = 0; $i < 10; $i++) {
            $password .= mt_rand();
        }

        $salt = substr(md5($password), 0, 2);

        $password = md5($salt . $plain) . ':' . $salt;

        return $password;
    }

    public function validatePassword($plain, $encrypt)
    {
        $stack = explode(':', $encrypt);
        if (sizeof($stack) != 2) {
            return false;
        }

        if (md5($stack[1] . $plain) == $stack[0]) {
            return true;
        }

        return false;
    }
}
